@@ -37,6 +37,14 @@ class CertificateConfigError(Exception):
         super(CertificateConfigError, self).__init__(msg)
 
 
+class CMSError(Exception):
+    """Error reading the certificate"""
+    def __init__(self, output):
+        self.output = output
+        msg = ("Unable to sign or verify data.")
+        super(CMSError, self).__init__(msg)
+
+
 class EmptyCatalog(EndpointNotFound):
     """The service catalog is empty."""
     pass
@@ -54,6 +62,10 @@ class VersionNotAvailable(DiscoveryFailure):
     """Discovery failed as the version you requested is not available."""
 
 
+class MethodNotImplemented(ClientException):
+    """Method not implemented by the keystoneclient API."""
+
+
 class MissingAuthPlugin(ClientException):
     """An authenticated request is required but no plugin available."""
 
@@ -61,3 +73,11 @@ class MissingAuthPlugin(ClientException):
 class NoMatchingPlugin(ClientException):
     """There were no auth plugins that could be created from the parameters
     provided."""
+
+
+class InvalidResponse(ClientException):
+    """The response from the server is not valid for this request."""
+
+    def __init__(self, response):
+        super(InvalidResponse, self).__init__()
+        self.response = response
