@@ -108,6 +108,8 @@ HTTP_X_ROLES
 
 HTTP_X_SERVICE_CATALOG
     json encoded keystone service catalog (optional).
+    For compatibility reasons this catalog will always be in the V2 catalog
+    format even if it is a v3 token.
 
 HTTP_X_TENANT_ID
     *Deprecated* in favor of HTTP_X_PROJECT_ID
@@ -164,7 +166,6 @@ from keystoneclient.middleware import memcache_crypt
 from keystoneclient.openstack.common import jsonutils
 from keystoneclient.openstack.common import memorycache
 from keystoneclient.openstack.common import timeutils
-from keystoneclient import utils
 
 
 # alternative middleware configuration in the main application's
@@ -221,7 +222,7 @@ opts = [
                default=None,
                help='Complete admin Identity API endpoint. This should '
                     'specify the unversioned root endpoint '
-                    'eg. https://localhost:35357/'),
+                    'e.g. https://localhost:35357/'),
     cfg.StrOpt('auth_version',
                default=None,
                help='API version of the admin Identity API endpoint'),
@@ -240,9 +241,12 @@ opts = [
                ' communicating with Identity API Server.'),
     cfg.StrOpt('admin_token',
                secret=True,
-               help='Single shared secret with the Keystone configuration'
+               help='This option is deprecated and may be removed in a future'
+               ' release. Single shared secret with the Keystone configuration'
                ' used for bootstrapping a Keystone installation, or otherwise'
-               ' bypassing the normal authentication process.'),
+               ' bypassing the normal authentication process. This option'
+               ' should not be used, use `admin_user` and `admin_password`'
+               ' instead.'),
     cfg.StrOpt('admin_user',
                help='Keystone account username'),
     cfg.StrOpt('admin_password',
@@ -277,7 +281,7 @@ opts = [
                ' configurable duration (in seconds). Set to -1 to disable'
                ' caching completely.'),
     cfg.IntOpt('revocation_cache_time',
-               default=300,
+               default=10,
                help='Determines the frequency at which the list of revoked'
                ' tokens is retrieved from the Identity service (in seconds). A'
                ' high number of revocation events combined with a low cache'
@@ -312,6 +316,20 @@ opts = [
                ' unknown the token will be rejected. "required" any form of'
                ' token binding is needed to be allowed. Finally the name of a'
                ' binding method that must be present in tokens.'),
+    cfg.BoolOpt('check_revocations_for_cached', default=False,
+                help='If true, the revocation list will be checked for cached'
+                ' tokens. This requires that PKI tokens are configured on the'
+                ' Keystone server.'),
+    cfg.ListOpt('hash_algorithms', default=['md5'],
+                help='Hash algorithms to use for hashing PKI tokens. This may'
+                ' be a single algorithm or multiple. The algorithms are those'
+                ' supported by Python standard hashlib.new(). The hashes will'
+                ' be tried in the order given, so put the preferred one first'
+                ' for performance. The result of the first hash will be stored'
+                ' in the cache. This will typically be set to multiple values'
+                ' only while migrating from a less secure algorithm to a more'
+                ' secure one. Once all the old tokens are expired this option'
+                ' should be set to a single value for better performance.'),
 ]
 
 CONF = cfg.CONF
@@ -364,6 +382,42 @@ def confirm_token_not_expired(data):
     return timeutils.isotime(at=expires, subsecond=True)
 
 
+def _v3_to_v2_catalog(catalog):
+    """Convert a catalog to v2 format.
+
+    X_SERVICE_CATALOG must be specified in v2 format. If you get a token
+    that is in v3 convert it.
+    """
+    v2_services = []
+    for v3_service in catalog:
+        # first copy over the entries we allow for the service
+        v2_service = {'type': v3_service['type']}
+        try:
+            v2_service['name'] = v3_service['name']
+        except KeyError:
+            pass
+
+        # now convert the endpoints. Because in v3 we specify region per
+        # URL not per group we have to collect all the entries of the same
+        # region together before adding it to the new service.
+        regions = {}
+        for v3_endpoint in v3_service.get('endpoints', []):
+            region_name = v3_endpoint.get('region')
+            try:
+                region = regions[region_name]
+            except KeyError:
+                region = {'region': region_name} if region_name else {}
+                regions[region_name] = region
+
+            interface_name = v3_endpoint['interface'].lower() + 'URL'
+            region[interface_name] = v3_endpoint['url']
+
+        v2_service['endpoints'] = list(regions.values())
+        v2_services.append(v2_service)
+
+    return v2_services
+
+
 def safe_quote(s):
     """URL-encode strings that are not already URL-encoded."""
     return urllib.parse.quote(s) if s == urllib.parse.unquote(s) else s
@@ -420,8 +474,9 @@ class AuthProtocol(object):
         # are backwards. We need to do it this way so that we can handle the
         # same deprecation strategy for CONF and the conf variable.
         if not self.identity_uri:
-            self.LOG.warning("Configuring admin URI using auth fragments. "
-                             "This is deprecated, use 'identity_uri' instead.")
+            self.LOG.warning('Configuring admin URI using auth fragments. '
+                             'This is deprecated, use \'identity_uri\''
+                             ' instead.')
 
             auth_host = self._conf_get('auth_host')
             auth_port = int(self._conf_get('auth_port'))
@@ -479,6 +534,12 @@ class AuthProtocol(object):
         # Credentials used to verify this component with the Auth service since
         # validating tokens is a privileged call
         self.admin_token = self._conf_get('admin_token')
+        if self.admin_token:
+            self.LOG.warning(
+                "The admin_token option in the auth_token middleware is "
+                "deprecated and should not be used. The admin_user and "
+                "admin_password options should be used instead. The "
+                "admin_token option may be removed in a future release.")
         self.admin_token_expiry = None
         self.admin_user = self._conf_get('admin_user')
         self.admin_password = self._conf_get('admin_password')
@@ -512,6 +573,9 @@ class AuthProtocol(object):
         self.include_service_catalog = self._conf_get(
             'include_service_catalog')
 
+        self.check_revocations_for_cached = self._conf_get(
+            'check_revocations_for_cached')
+
     def _assert_valid_memcache_protection_config(self):
         if self._memcache_security_strategy:
             if self._memcache_security_strategy not in ('MAC', 'ENCRYPT'):
@@ -572,8 +636,8 @@ class AuthProtocol(object):
         versions = []
         response, data = self._json_request('GET', '/')
         if response.status_code == 501:
-            self.LOG.warning("Old keystone installation found...assuming v2.0")
-            versions.append("v2.0")
+            self.LOG.warning('Old keystone installation found...assuming v2.0')
+            versions.append('v2.0')
         elif response.status_code != 300:
             self.LOG.error('Unable to get version info from keystone: %s',
                            response.status_code)
@@ -675,9 +739,9 @@ class AuthProtocol(object):
             return token
         else:
             if not self.delay_auth_decision:
-                self.LOG.warn("Unable to find authentication token"
-                              " in headers")
-                self.LOG.debug("Headers: %s", env)
+                self.LOG.warn('Unable to find authentication token'
+                              ' in headers')
+                self.LOG.debug('Headers: %s', env)
             raise InvalidUserToken('Unable to find token in headers')
 
     def _reject_request(self, env, start_response):
@@ -723,7 +787,7 @@ class AuthProtocol(object):
         :raise ServerError when unable to communicate with keystone
 
         """
-        url = "%s/%s" % (self.identity_uri, path.lstrip('/'))
+        url = '%s/%s' % (self.identity_uri, path.lstrip('/'))
 
         kwargs.setdefault('timeout', self.http_connect_timeout)
         if self.cert_file and self.key_file:
@@ -822,16 +886,16 @@ class AuthProtocol(object):
             return (token, timeutils.normalize_time(datetime_expiry))
         except (AssertionError, KeyError):
             self.LOG.warn(
-                "Unexpected response from keystone service: %s", data)
+                'Unexpected response from keystone service: %s', data)
             raise ServiceError('invalid json response')
         except (ValueError):
             data['access']['token']['id'] = '<SANITIZED>'
             self.LOG.warn(
-                "Unable to parse expiration time from token: %s", data)
+                'Unable to parse expiration time from token: %s', data)
             raise ServiceError('invalid json response')
 
     def _validate_user_token(self, user_token, env, retry=True):
-        """Authenticate user using PKI
+        """Authenticate user token
 
         :param user_token: user's token id
         :param retry: Ignored, as it is not longer relevant
@@ -843,12 +907,27 @@ class AuthProtocol(object):
         token_id = None
 
         try:
-            token_id = cms.cms_hash_token(user_token)
-            cached = self._cache_get(token_id)
+            token_ids, cached = self._check_user_token_cached(user_token)
+            token_id = token_ids[0]
             if cached:
-                return cached
-            if cms.is_asn1_token(user_token):
-                verified = self.verify_signed_token(user_token)
+                data = cached
+
+                if self.check_revocations_for_cached:
+                    # A token stored in Memcached might have been revoked
+                    # regardless of initial mechanism used to validate it,
+                    # and needs to be checked.
+                    for tid in token_ids:
+                        is_revoked = self._is_token_id_in_revoked_list(tid)
+                        if is_revoked:
+                            self.LOG.debug(
+                                'Token is marked as having been revoked')
+                            raise InvalidUserToken(
+                                'Token authorization failed')
+            elif cms.is_pkiz(user_token):
+                verified = self.verify_pkiz_token(user_token, token_ids)
+                data = jsonutils.loads(verified)
+            elif cms.is_asn1_token(user_token):
+                verified = self.verify_signed_token(user_token, token_ids)
                 data = jsonutils.loads(verified)
             else:
                 data = self.verify_uuid_token(user_token, retry)
@@ -858,15 +937,48 @@ class AuthProtocol(object):
             return data
         except NetworkError:
             self.LOG.debug('Token validation failure.', exc_info=True)
-            self.LOG.warn("Authorization failed for token")
+            self.LOG.warn('Authorization failed for token')
             raise InvalidUserToken('Token authorization failed')
         except Exception:
             self.LOG.debug('Token validation failure.', exc_info=True)
             if token_id:
                 self._cache_store_invalid(token_id)
-            self.LOG.warn("Authorization failed for token")
+            self.LOG.warn('Authorization failed for token')
             raise InvalidUserToken('Token authorization failed')
 
+    def _check_user_token_cached(self, user_token):
+        """Check if the token is cached already.
+
+        Returns a tuple. The first element is a list of token IDs, where the
+        first one is the preferred hash.
+
+        The second element is the token data from the cache if the token was
+        cached, otherwise ``None``.
+
+        :raises InvalidUserToken: if the token is invalid
+
+        """
+
+        if cms.is_asn1_token(user_token):
+            # user_token is a PKI token that's not hashed.
+
+            algos = self._conf_get('hash_algorithms')
+            token_hashes = list(cms.cms_hash_token(user_token, mode=algo)
+                                for algo in algos)
+
+            for token_hash in token_hashes:
+                cached = self._cache_get(token_hash)
+                if cached:
+                    return (token_hashes, cached)
+
+            # The token wasn't found using any hash algorithm.
+            return (token_hashes, None)
+
+        # user_token is either a UUID token or a hashed PKI token.
+        token_id = user_token
+        cached = self._cache_get(token_id)
+        return ([token_id], cached)
+
     def _build_user_headers(self, token_info):
         """Convert token object into headers.
 
@@ -878,7 +990,7 @@ class AuthProtocol(object):
 
         """
         auth_ref = access.AccessInfo.factory(body=token_info)
-        roles = ",".join(auth_ref.role_names)
+        roles = ','.join(auth_ref.role_names)
 
         if _token_is_v2(token_info) and not auth_ref.project_id:
             raise InvalidUserToken('Unable to determine tenancy.')
@@ -904,12 +1016,14 @@ class AuthProtocol(object):
             'X-Role': roles,
         }
 
-        self.LOG.debug("Received request from user: %s with project_id : %s"
-                       " and roles: %s ",
+        self.LOG.debug('Received request from user: %s with project_id : %s'
+                       ' and roles: %s ',
                        auth_ref.user_id, auth_ref.project_id, roles)
 
         if self.include_service_catalog and auth_ref.has_service_catalog():
             catalog = auth_ref.service_catalog.get_data()
+            if _token_is_v3(token_info):
+                catalog = _v3_to_v2_catalog(catalog)
             rval['X-Service-Catalog'] = jsonutils.dumps(catalog)
 
         return rval
@@ -943,7 +1057,7 @@ class AuthProtocol(object):
         env_key = self._header_to_env_var(key)
         return env.get(env_key, default)
 
-    def _cache_get(self, token_id, ignore_expires=False):
+    def _cache_get(self, token_id):
         """Return token information from cache.
 
         If token is invalid raise InvalidUserToken
@@ -1006,7 +1120,7 @@ class AuthProtocol(object):
 
             expires = timeutils.normalize_time(expires)
             utcnow = timeutils.utcnow()
-            if ignore_expires or utcnow < expires:
+            if utcnow < expires:
                 self.LOG.debug('Returning cached token')
                 return data
             else:
@@ -1070,7 +1184,7 @@ class AuthProtocol(object):
                 # no bind provided and none required
                 return
             else:
-                self.LOG.info("No bind information present in token.")
+                self.LOG.info('No bind information present in token.')
                 self._invalid_user_token()
 
         # get the named mode if bind_mode is not one of the predefined
@@ -1080,32 +1194,32 @@ class AuthProtocol(object):
             name = bind_mode
 
         if name and name not in bind:
-            self.LOG.info("Named bind mode %s not in bind information", name)
+            self.LOG.info('Named bind mode %s not in bind information', name)
             self._invalid_user_token()
 
         for bind_type, identifier in six.iteritems(bind):
             if bind_type == BIND_MODE.KERBEROS:
                 if not env.get('AUTH_TYPE', '').lower() == 'negotiate':
-                    self.LOG.info("Kerberos credentials required and "
-                                  "not present.")
+                    self.LOG.info('Kerberos credentials required and '
+                                  'not present.')
                     self._invalid_user_token()
 
                 if not env.get('REMOTE_USER') == identifier:
-                    self.LOG.info("Kerberos credentials do not match "
-                                  "those in bind.")
+                    self.LOG.info('Kerberos credentials do not match '
+                                  'those in bind.')
                     self._invalid_user_token()
 
-                self.LOG.debug("Kerberos bind authentication successful.")
+                self.LOG.debug('Kerberos bind authentication successful.')
 
             elif bind_mode == BIND_MODE.PERMISSIVE:
-                self.LOG.debug("Ignoring Unknown bind for permissive mode: "
-                               "%(bind_type)s: %(identifier)s.",
+                self.LOG.debug('Ignoring Unknown bind for permissive mode: '
+                               '%(bind_type)s: %(identifier)s.',
                                {'bind_type': bind_type,
                                 'identifier': identifier})
 
             else:
-                self.LOG.info("Couldn't verify unknown bind: %(bind_type)s: "
-                              "%(identifier)s.",
+                self.LOG.info('Couldn`t verify unknown bind: %(bind_type)s: '
+                              '%(identifier)s.',
                               {'bind_type': bind_type,
                                'identifier': identifier})
                 self._invalid_user_token()
@@ -1162,7 +1276,7 @@ class AuthProtocol(object):
         if response.status_code == 200:
             return data
         if response.status_code == 404:
-            self.LOG.warn("Authorization failed for token")
+            self.LOG.warn('Authorization failed for token')
             raise InvalidUserToken('Token authorization failed')
         if response.status_code == 401:
             self.LOG.info(
@@ -1175,27 +1289,29 @@ class AuthProtocol(object):
             self.LOG.info('Retrying validation')
             return self.verify_uuid_token(user_token, False)
         else:
-            self.LOG.warn("Invalid user token. Keystone response: %s", data)
+            self.LOG.warn('Invalid user token. Keystone response: %s', data)
 
             raise InvalidUserToken()
 
-    def is_signed_token_revoked(self, signed_text):
+    def is_signed_token_revoked(self, token_ids):
         """Indicate whether the token appears in the revocation list."""
-        revocation_list = self.token_revocation_list
-        revoked_tokens = revocation_list.get('revoked', [])
-        if not revoked_tokens:
-            return
-        revoked_ids = (x['id'] for x in revoked_tokens)
-        if isinstance(signed_text, six.text_type):
-            signed_text = signed_text.encode('utf-8')
-        token_id = utils.hash_signed_token(signed_text)
-        for revoked_id in revoked_ids:
-            if token_id == revoked_id:
+        for token_id in token_ids:
+            if self._is_token_id_in_revoked_list(token_id):
                 self.LOG.debug('Token is marked as having been revoked')
                 return True
         return False
 
-    def cms_verify(self, data):
+    def _is_token_id_in_revoked_list(self, token_id):
+        """Indicate whether the token_id appears in the revocation list."""
+        revocation_list = self.token_revocation_list
+        revoked_tokens = revocation_list.get('revoked', None)
+        if not revoked_tokens:
+            return False
+
+        revoked_ids = (x['id'] for x in revoked_tokens)
+        return token_id in revoked_ids
+
+    def cms_verify(self, data, inform=cms.PKI_ASN1_FORM):
         """Verifies the signature of the provided data's IAW CMS syntax.
 
         If either of the certificate files might be missing, fetch them and
@@ -1204,7 +1320,8 @@ class AuthProtocol(object):
         def verify():
             try:
                 return cms.cms_verify(data, self.signing_cert_file_name,
-                                      self.signing_ca_file_name)
+                                      self.signing_ca_file_name,
+                                      inform=inform).decode('utf-8')
             except cms.subprocess.CalledProcessError as err:
                 self.LOG.warning('Verify error: %s', err)
                 raise
@@ -1225,13 +1342,25 @@ class AuthProtocol(object):
                 self.LOG.error('CMS Verify output: %s', err.output)
                 raise
 
-    def verify_signed_token(self, signed_text):
+    def verify_signed_token(self, signed_text, token_ids):
         """Check that the token is unrevoked and has a valid signature."""
-        if self.is_signed_token_revoked(signed_text):
+        if self.is_signed_token_revoked(token_ids):
             raise InvalidUserToken('Token has been revoked')
 
         formatted = cms.token_to_cms(signed_text)
-        return self.cms_verify(formatted)
+        verified = self.cms_verify(formatted)
+        return verified
+
+    def verify_pkiz_token(self, signed_text, token_ids):
+        if self.is_signed_token_revoked(token_ids):
+            raise InvalidUserToken('Token has been revoked')
+        try:
+            uncompressed = cms.pkiz_uncompress(signed_text)
+            verified = self.cms_verify(uncompressed, inform=cms.PKIZ_CMS_FORM)
+            return verified
+        # TypeError If the signed_text is not zlib compressed
+        except TypeError:
+            raise InvalidUserToken(signed_text)
 
     def verify_signing_dir(self):
         if os.path.exists(self.signing_dirname):
