@@ -115,7 +115,7 @@ class Session(object):
     def request(self, url, method, json=None, original_ip=None,
                 user_agent=None, redirect=None, authenticated=None,
                 endpoint_filter=None, auth=None, requests_auth=None,
-                raise_exc=True, **kwargs):
+                raise_exc=True, allow_reauth=True, **kwargs):
         """Send an HTTP request with the specified characteristics.
 
         Wrapper around `requests.Session.request` to handle tasks such as
@@ -129,7 +129,7 @@ class Session(object):
                            provided such that the base URL can be determined.
                            If a fully qualified URL is provided then
                            endpoint_filter will be ignored.
-        :param string method: The http method to use. (eg. 'GET', 'POST')
+        :param string method: The http method to use. (e.g. 'GET', 'POST')
         :param string original_ip: Mark this request as forwarded for this ip.
                                    (optional)
         :param dict headers: Headers to be included in the request. (optional)
@@ -161,6 +161,9 @@ class Session(object):
         :param bool raise_exc: If True then raise an appropriate exception for
                                failed HTTP requests. If False then return the
                                request object. (optional, default True)
+        :param bool allow_reauth: Allow fetching a new token and retrying the
+                                  request on receiving a 401 Unauthorized
+                                  response. (optional, default True)
         :param kwargs: any other parameter that can be passed to
                        requests.Session.request (such as `headers`). Except:
                        'data' will be overwritten by the data in 'json' param.
@@ -186,7 +189,7 @@ class Session(object):
 
             headers['X-Auth-Token'] = token
 
-        # if we are passed a fully qualified URL and a endpoint_filter we
+        # if we are passed a fully qualified URL and an endpoint_filter we
         # should ignore the filter. This will make it easier for clients who
         # want to overrule the default endpoint_filter data added to all client
         # requests. We check fully qualified here by the presence of a host.
@@ -256,9 +259,14 @@ class Session(object):
 
         resp = self._send_request(url, method, redirect, **kwargs)
 
-        # NOTE(jamielennox): we create a tuple here to be the same as what is
-        # returned by the requests library.
-        resp.history = tuple(resp.history)
+        # handle getting a 401 Unauthorized response by invalidating the plugin
+        # and then retrying the request. This is only tried once.
+        if resp.status_code == 401 and authenticated and allow_reauth:
+            if self.invalidate(auth):
+                token = self.get_token(auth)
+                if token:
+                    headers['X-Auth-Token'] = token
+                    resp = self._send_request(url, method, redirect, **kwargs)
 
         if raise_exc and resp.status_code >= 400:
             _logger.debug('Request returned failure status: %s',
@@ -412,3 +420,15 @@ class Session(object):
                                                'determine the endpoint URL.')
 
         return auth.get_endpoint(self, **kwargs)
+
+    def invalidate(self, auth=None):
+        """Invalidate an authentication plugin.
+        """
+        if not auth:
+            auth = self.auth
+
+        if not auth:
+            msg = 'Auth plugin not available to invalidate'
+            raise exceptions.MissingAuthPlugin(msg)
+
+        return auth.invalidate()
